@@ -647,7 +647,7 @@ static const struct {
 static const struct {
 	const char *icon, *shortdesc, *description;
 } smime_encrypt_table[4] = {
-	{ "stock_lock-broken", N_("Unencrypted"), N_("This message is not encrypted.  Its content may be viewed in transit across The Internet.") },
+	{ "stock_lock-broken", N_("Unencrypted"), N_("This message is not encrypted.  Its content may be viewed in transit across the Internet.") },
 	{ "stock_lock-ok", N_("Encrypted, weak"), N_("This message is encrypted, but with a weak encryption algorithm.  It would be difficult, but not impossible for an outsider to view the content of this message in a practical amount of time.") },
 	{ "stock_lock-ok", N_("Encrypted"), N_("This message is encrypted.  It would be difficult for an outsider to view the content of this message.") },
 	{ "stock_lock-ok", N_("Encrypted, strong"), N_("This message is encrypted, with a strong encryption algorithm.  It would be very difficult for an outsider to view the content of this message in a practical amount of time.") },
@@ -1046,38 +1046,7 @@ efhd_attachment_show(GtkWidget *w, struct _attach_puri *info)
 	d(printf("show attachment button called\n"));
 
 	info->shown = ~info->shown;
-	em_format_set_inline(info->puri.format, info->puri.part, info->shown);
-	/* FIXME: do this in an idle handler */
-	em_format_redraw(info->puri.format);
-#if 0
-	/* FIXME: track shown state in parent */
-
-	if (info->shown) {
-		d(printf("hiding\n"));
-		info->shown = FALSE;
-		if (info->frame)
-			gtk_widget_hide((GtkWidget *)info->frame);
-		gtk_widget_show(info->forward);
-		gtk_widget_hide(info->down);
-	} else {
-		d(printf("showing\n"));
-		info->shown = TRUE;
-		if (info->frame)
-			gtk_widget_show((GtkWidget *)info->frame);
-		gtk_widget_hide(info->forward);
-		gtk_widget_show(info->down);
-
-		/* have we decoded it yet? */
-		if (info->output) {
-			info->handle->handler(info->puri.format, info->output, info->puri.part, info->handle);
-			camel_stream_close(info->output);
-			camel_object_unref(info->output);
-			info->output = NULL;
-		}
-	}
-
-	em_format_set_inline(info->puri.format, info->puri.part, info->shown);
-#endif
+	em_format_set_inline(info->puri.format, info->puri.part_id, info->shown);
 }
 
 static EMPopupItem efhd_menu_items[] = {
@@ -1404,7 +1373,13 @@ efhd_bonobo_object(EMFormatHTML *efh, GtkHTMLEmbedded *eb, EMFormatHTMLPObject *
 	/* Write the data to a CamelStreamMem... */
 	cstream = (CamelStreamMem *)camel_stream_mem_new();
 	wrapper = camel_medium_get_content_object((CamelMedium *)pobject->part);
- 	camel_data_wrapper_decode_to_stream(wrapper, (CamelStream *)cstream);
+	if (FALSE && !g_ascii_strncasecmp (eb->type, "text/", 5)) {
+		/* do charset conversion, etc */
+		printf ("performing charset conversion for %s component\n", eb->type);
+		em_format_format_text ((EMFormat *) efh, (CamelStream *) cstream, wrapper);
+	} else {
+		camel_data_wrapper_decode_to_stream (wrapper, (CamelStream *) cstream);
+	}
 	
 	/* ...convert the CamelStreamMem to a BonoboStreamMem... */
 	bstream = bonobo_stream_mem_create(cstream->buffer->data, cstream->buffer->len, TRUE, FALSE);
@@ -1498,7 +1473,7 @@ efhd_format_attachment(EMFormat *emf, CamelStream *stream, CamelMimePart *part,
 	info = (struct _attach_puri *)em_format_add_puri(emf, sizeof(*info), classid, part, efhd_attachment_frame);
 	em_format_html_add_pobject((EMFormatHTML *)emf, sizeof(EMFormatHTMLPObject), classid, part, efhd_attachment_button);
 	info->handle = handle;
-	info->shown = em_format_is_inline(emf, info->puri.part, handle);
+	info->shown = em_format_is_inline(emf, info->puri.part_id, info->puri.part, handle);
 	info->snoop_mime_type = emf->snoop_mime_type;
 
 	camel_stream_write_string(stream,
