@@ -629,6 +629,7 @@ importer_file_page_new (ImportData *data)
 	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
 
 	page->filename = gnome_file_entry_new ("Evolution_Importer_FileName", _("Select a file"));
+	g_object_set (G_OBJECT (page->filename), "use_filechooser", TRUE, NULL);
 	entry = gnome_file_entry_gtk_entry((GnomeFileEntry *)page->filename);
 	g_signal_connect (entry, "changed", G_CALLBACK (filename_changed), data);
 	gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
@@ -1177,7 +1178,7 @@ e_shell_importer_start_import (EShellWindow *shell_window)
 		return;
 	}
 	
-	icon = e_icon_factory_get_icon ("stock_import", E_ICON_SIZE_DIALOG);
+	icon = e_icon_factory_get_icon ("stock_mail-import", E_ICON_SIZE_DIALOG);
 	
 	dialog_open = TRUE;
 	data->window = shell_window;
